syntax = "proto3";

package ttrpc;

option go_package = "github.com/containerd/ttrpc/internal";

message TestPayload {
	string foo = 1;
	int64 deadline = 2;
	string metadata = 3;
}

message EchoPayload {
	int64 seq = 1;
	string msg = 2;
}
