/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type SecuritySettings struct {
	// Enablement of Trusted Platform Module on the computer system
	EnableTpm bool               `json:"EnableTpm,omitempty"`
	Isolation *IsolationSettings `json:"Isolation,omitempty"`
}
