/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type DeviceInstance struct {
	Id             string           `json:"id,omitempty"`
	LocationPath   string           `json:"location_path,omitempty"`
	PortName       string           `json:"port_name,omitempty"`
	InterfaceClass []InterfaceClass `json:"interface_class,omitempty"`
}
