/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type ObjectSymlink struct {
	Name        string `json:"name,omitempty"`
	Path        string `json:"path,omitempty"`
	Scope       string `json:"scope,omitempty"`
	Pathtoclone string `json:"pathtoclone,omitempty"`
	AccessMask  int32  `json:"access_mask,omitempty"`
}
