// Code generated by "stringer -type=Level -trimprefix=Level"; DO NOT EDIT.

package etw

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[LevelAlways-0]
	_ = x[LevelCritical-1]
	_ = x[LevelError-2]
	_ = x[LevelWarning-3]
	_ = x[LevelInfo-4]
	_ = x[LevelVerbose-5]
}

const _Level_name = "AlwaysCriticalErrorWarningInfoVerbose"

var _Level_index = [...]uint8{0, 6, 14, 19, 26, 30, 37}

func (i Level) String() string {
	if i >= Level(len(_Level_index)-1) {
		return "Level(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _Level_name[_Level_index[i]:_Level_index[i+1]]
}
