/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.1
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type NetworkAdapter struct {
	EndpointId string `json:"EndpointId,omitempty"`
	MacAddress string `json:"MacAddress,omitempty"`
	// The I/O virtualization (IOV) offloading configuration.
	IovSettings *IovSettings `json:"IovSettings,omitempty"`
}
