/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type IsolationSettings struct {
	// Guest isolation type options to decide virtual trust levels of virtual machine
	IsolationType string `json:"IsolationType,omitempty"`
	// Configuration to debug HCL layer for HCS VM TODO: Task 31102306: Miss the way to prevent the exposure of private debug configuration in HCS TODO: Think about the secret configurations which are private in VMMS VM (only edit by hvsedit)
	DebugHost string `json:"DebugHost,omitempty"`
	DebugPort int64  `json:"DebugPort,omitempty"`
	// Optional data passed by host on isolated virtual machine start
	LaunchData string `json:"LaunchData,omitempty"`
	HclEnabled bool   `json:"HclEnabled,omitempty"`
}
