/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type ContainerCredentialGuardInstance struct {
	Id              string                                         `json:"Id,omitempty"`
	CredentialGuard *ContainerCredentialGuardState                 `json:"CredentialGuard,omitempty"`
	HvSocketConfig  *ContainerCredentialGuardHvSocketServiceConfig `json:"HvSocketConfig,omitempty"`
}
