/*
 * HCS API
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 2.4
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package hcsschema

type CPUGroupPropertyCode uint32

const (
	CPUCapacityProperty           = 0x00010000
	CPUSchedulingPriorityProperty = 0x00020000
	IdleLPReserveProperty         = 0x00030000
)

type CpuGroupProperty struct {
	PropertyCode  uint32 `json:"PropertyCode,omitempty"`
	PropertyValue uint32 `json:"PropertyValue,omitempty"`
}
